@@ -22,6 +22,9 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+
+#include "config.h"
+
 #include <math.h>
 #include <string.h>
 #include <stdio.h>
@@ -59,6 +62,7 @@ struct buffer {
 };
 
 struct data {
+	const char *plugin_dir;
 	struct spa_log *log;
 	struct spa_system *system;
 	struct spa_loop *loop;
@@ -95,14 +99,22 @@ static int load_handle(struct data *data, struct spa_handle **handle, const char
 	void *hnd;
 	spa_handle_factory_enum_func_t enum_func;
 	uint32_t i;
+	char *path;
 
-	if ((hnd = dlopen(lib, RTLD_NOW)) == NULL) {
+	if ((path = spa_aprintf("%s/%s", data->plugin_dir, lib)) == NULL)
+		return -ENOMEM;
+
+	hnd = dlopen(path, RTLD_NOW);
+	free(path);
+
+	if (hnd == NULL) {
 		printf("can't load %s: %s\n", lib, dlerror());
-		return -errno;
+		return -ENOENT;
 	}
 	if ((enum_func = dlsym(hnd, SPA_HANDLE_FACTORY_ENUM_FUNC_NAME)) == NULL) {
 		printf("can't find enum function\n");
-		return -errno;
+		res = -ENOENT;
+		goto exit_cleanup;
 	}
 
 	for (i = 0;;) {
@@ -113,6 +125,8 @@ static int load_handle(struct data *data, struct spa_handle **handle, const char
 				printf("can't enumerate factories: %s\n", spa_strerror(res));
 			break;
 		}
+		if (factory->version < 1)
+			continue;
 		if (strcmp(factory->name, name))
 			continue;
 
@@ -121,11 +135,15 @@ static int load_handle(struct data *data, struct spa_handle **handle, const char
 						NULL, data->support,
 						data->n_support)) < 0) {
 			printf("can't make factory instance: %d\n", res);
-			return res;
+			goto exit_cleanup;
 		}
 		return 0;
 	}
 	return -EBADF;
+
+exit_cleanup:
+	dlclose(hnd);
+	return res;
 }
 
 int init_data(struct data *data)
@@ -135,6 +153,10 @@ int init_data(struct data *data)
 	struct spa_handle *handle = NULL;
 	void *iface;
 
+	if ((str = getenv("SPA_PLUGIN_DIR")) == NULL)
+		str = PLUGINDIR;
+	data->plugin_dir = str;
+
 	/* init the graph */
 	spa_graph_init(&data->graph, &data->graph_state);
 
@@ -144,7 +166,7 @@ int init_data(struct data *data)
 
 	/* load and set support system */
 	if ((res = load_handle(data, &handle,
-			"build/spa/plugins/support/libspa-support.so",
+			"support/libspa-support.so",
 			SPA_NAME_SUPPORT_SYSTEM)) < 0)
 		return res;
 	if ((res = spa_handle_get_interface(handle, SPA_TYPE_INTERFACE_System, &iface)) < 0) {
@@ -157,7 +179,7 @@ int init_data(struct data *data)
 
 	/* load and set support loop and loop control */
 	if ((res = load_handle(data, &handle,
-			"build/spa/plugins/support/libspa-support.so",
+			"support/libspa-support.so",
 			SPA_NAME_SUPPORT_LOOP)) < 0)
 		return res;
 
@@ -185,17 +207,25 @@ static int make_node(struct data *data, struct spa_node **node, const char *lib,
 {
 	struct spa_handle *handle;
 	int res = 0;
-	void *hnd;
+	void *hnd = NULL;
 	spa_handle_factory_enum_func_t enum_func;
 	uint32_t i;
+	char *path;
 
-	if ((hnd = dlopen(lib, RTLD_NOW)) == NULL) {
+	if ((path = spa_aprintf("%s/%s", data->plugin_dir, lib)) == NULL)
+		return -ENOMEM;
+
+	hnd = dlopen(path, RTLD_NOW);
+	free(path);
+
+	if (hnd == NULL) {
 		printf("can't load %s: %s\n", lib, dlerror());
-		return -errno;
+		return -ENOENT;
 	}
 	if ((enum_func = dlsym(hnd, SPA_HANDLE_FACTORY_ENUM_FUNC_NAME)) == NULL) {
 		printf("can't find enum function\n");
-		return -errno;
+		res = -ENOENT;
+		goto exit_cleanup;
 	}
 
 	for (i = 0;;) {
@@ -207,6 +237,8 @@ static int make_node(struct data *data, struct spa_node **node, const char *lib,
 				printf("can't enumerate factories: %s\n", spa_strerror(res));
 			break;
 		}
+		if (factory->version < 1)
+			continue;
 		if (strcmp(factory->name, name))
 			continue;
 
@@ -215,16 +247,20 @@ static int make_node(struct data *data, struct spa_node **node, const char *lib,
 		     spa_handle_factory_init(factory, handle, props, data->support,
 					     data->n_support)) < 0) {
 			printf("can't make factory instance: %d\n", res);
-			return res;
+			goto exit_cleanup;
 		}
 		if ((res = spa_handle_get_interface(handle, SPA_TYPE_INTERFACE_Node, &iface)) < 0) {
 			printf("can't get interface %d\n", res);
-			return res;
+			goto exit_cleanup;
 		}
 		*node = iface;
 		return 0;
 	}
 	return -EBADF;
+
+exit_cleanup:
+	dlclose(hnd);
+	return res;
 }
 
 static int on_sink_node_ready(void *_data, int status)
@@ -265,7 +301,7 @@ static int make_nodes(struct data *data, const char *device)
 
 	/* make the source node (audiotestsrc) */
 	if ((res = make_node(data, &data->source_follower_node,
-				   "build/spa/plugins/audiotestsrc/libspa-audiotestsrc.so",
+				   "audiotestsrc/libspa-audiotestsrc.so",
 				   "audiotestsrc",
 				   NULL)) < 0) {
 		printf("can't create source follower node (audiotestsrc): %d\n", res);
@@ -290,7 +326,7 @@ static int make_nodes(struct data *data, const char *device)
 	snprintf(value, sizeof(value), "pointer:%p", data->source_follower_node);
 	items[0] = SPA_DICT_ITEM_INIT("audio.adapt.follower", value);
 	if ((res = make_node(data, &data->source_node,
-			     "build/spa/plugins/audioconvert/libspa-audioconvert.so",
+			     "audioconvert/libspa-audioconvert.so",
 			     SPA_NAME_AUDIO_ADAPT,
 			     &SPA_DICT_INIT(items, 1))) < 0) {
 		printf("can't create source adapter node: %d\n", res);
@@ -329,7 +365,7 @@ static int make_nodes(struct data *data, const char *device)
 
 	/* make the sink follower node (alsa-pcm-sink) */
 	if ((res = make_node(data, &data->sink_follower_node,
-				   "build/spa/plugins/alsa/libspa-alsa.so",
+				   "alsa/libspa-alsa.so",
 				   SPA_NAME_API_ALSA_PCM_SINK,
 				   NULL)) < 0) {
 		printf("can't create sink follower node (alsa-pcm-sink): %d\n", res);
@@ -340,7 +376,7 @@ static int make_nodes(struct data *data, const char *device)
 	snprintf(value, sizeof(value), "pointer:%p", data->sink_follower_node);
 	items[0] = SPA_DICT_ITEM_INIT("audio.adapt.follower", value);
 	if ((res = make_node(data, &data->sink_node,
-			     "build/spa/plugins/audioconvert/libspa-audioconvert.so",
+			     "audioconvert/libspa-audioconvert.so",
 			     SPA_NAME_AUDIO_ADAPT,
 			     &SPA_DICT_INIT(items, 1))) < 0) {
 		printf("can't create sink adapter node: %d\n", res);
@@ -466,10 +502,10 @@ static int negotiate_formats(struct data *data)
 	if (spa_node_port_enum_params_sync(data->source_follower_node,
 			SPA_DIRECTION_OUTPUT, 0,
 			SPA_PARAM_Buffers, &state, filter, &param, &b) != 1)
-		return res;
+		return -ENOTSUP;
 	spa_pod_fixate(param);
-	if (spa_pod_parse_object(param, SPA_TYPE_OBJECT_ParamBuffers, NULL,
-		SPA_PARAM_BUFFERS_size, SPA_POD_Int(&buffer_size)) < 0)
+	if ((res = spa_pod_parse_object(param, SPA_TYPE_OBJECT_ParamBuffers, NULL,
+		SPA_PARAM_BUFFERS_size, SPA_POD_Int(&buffer_size))) < 0)
 		return res;
 
 	/* set the sink and source formats */
