@@ -226,7 +226,7 @@ static inline void clean_name(char *name)
 {
 	char *c;
 	for (c = name; *c; ++c) {
-		if (!isalnum(*c) && strchr(" /_:()", *c) == NULL)
+		if (!isalnum(*c) && strchr(" /_:()[]", *c) == NULL)
 			*c = '-';
 	}
 }
@@ -254,10 +254,12 @@ static void emit_port_info(struct seq_state *this, struct seq_port *port, bool f
 		snd_seq_get_any_client_info(this->sys.hndl,
 				port->addr.client, client_info);
 
-		snprintf(name, sizeof(name), "%s:%s_%d",
+		snprintf(name, sizeof(name), "%s [%d] (%s): [%d] %s",
 				snd_seq_client_info_get_name(client_info),
+				port->addr.client,
 				port->direction == SPA_DIRECTION_OUTPUT ? "capture" : "playback",
-				port->addr.port);
+				port->addr.port,
+				snd_seq_port_info_get_name(info));
 		clean_name(name);
 
 		snprintf(path, sizeof(path), "alsa:seq:%s:client_%d:%s_%d",
@@ -391,16 +393,17 @@ static void free_port(struct seq_state *state, struct seq_stream *stream, struct
 	spa_zero(*port);
 }
 
-static void init_port(struct seq_state *state, struct seq_port *port, const snd_seq_addr_t *addr)
+static void init_port(struct seq_state *state, struct seq_port *port, const snd_seq_addr_t *addr,
+		unsigned int caps)
 {
 	port->addr = *addr;
 	port->info_all = SPA_PORT_CHANGE_MASK_FLAGS |
 			SPA_PORT_CHANGE_MASK_PROPS |
 			SPA_PORT_CHANGE_MASK_PARAMS;
 	port->info = SPA_PORT_INFO_INIT();
-	port->info.flags = SPA_PORT_FLAG_LIVE |
-			   SPA_PORT_FLAG_PHYSICAL |
-			   SPA_PORT_FLAG_TERMINAL;
+	port->info.flags = SPA_PORT_FLAG_LIVE;
+	if (caps & (SND_SEQ_PORT_TYPE_HARDWARE|SND_SEQ_PORT_TYPE_PORT|SND_SEQ_PORT_TYPE_SPECIFIC))
+		port->info.flags |= SPA_PORT_FLAG_PHYSICAL | SPA_PORT_FLAG_TERMINAL;
 	port->params[0] = SPA_PARAM_INFO(SPA_PARAM_EnumFormat, SPA_PARAM_INFO_READ);
 	port->params[1] = SPA_PARAM_INFO(SPA_PARAM_Meta, SPA_PARAM_INFO_READ);
 	port->params[2] = SPA_PARAM_INFO(SPA_PARAM_IO, SPA_PARAM_INFO_READ);
@@ -433,7 +436,7 @@ static void update_stream_port(struct seq_state *state, struct seq_stream *strea
 			port = alloc_port(state, stream);
 			if (port == NULL)
 				return;
-			init_port(state, port, addr);
+			init_port(state, port, addr, caps);
 		} else if (port != NULL) {
 			if ((caps & stream->caps) != stream->caps) {
 				spa_log_debug(state->log, "free port %d.%d", addr->client, addr->port);
