@@ -83,6 +83,10 @@ struct sm_object {
 	struct spa_callbacks methods;
 
 	struct spa_list data;
+
+	unsigned int monitor_global:1;	/**< whether handle is from monitor core */
+	unsigned int destroyed:1;	/**< whether proxies have been destroyed */
+	unsigned int discarded:1;	/**< whether monitors hold no references */
 };
 
 int sm_object_add_listener(struct sm_object *obj, struct spa_hook *listener,
@@ -109,6 +113,8 @@ int sm_object_sync_update(struct sm_object *obj);
 
 int sm_object_destroy(struct sm_object *obj);
 
+#define sm_object_discard(o)	do { (o)->discarded = true; } while (0)
+
 struct sm_client {
 	struct sm_object obj;
 
@@ -293,9 +299,9 @@ int sm_media_session_load_conf(struct sm_media_session *sess,
 		const char *name, struct pw_properties *conf);
 
 int sm_media_session_load_state(struct sm_media_session *sess,
-		const char *name, const char *prefix, struct pw_properties *props);
+		const char *name, struct pw_properties *props);
 int sm_media_session_save_state(struct sm_media_session *sess,
-		const char *name, const char *prefix, const struct pw_properties *props);
+		const char *name, const struct pw_properties *props);
 
 int sm_media_session_match_rules(const char *rules, size_t size,
 		struct pw_properties *props);
