@@ -93,10 +93,12 @@ void pw_manager_add_listener(struct pw_manager *manager,
 		struct spa_hook *listener,
 		const struct pw_manager_events *events, void *data);
 
+int pw_manager_sync(struct pw_manager *manager);
+
 void pw_manager_destroy(struct pw_manager *manager);
 
 int pw_manager_set_metadata(struct pw_manager *manager,
-		struct pw_manager_object *metdata,
+		struct pw_manager_object *metadata,
 		uint32_t subject, const char *key, const char *type,
 		const char *format, ...) SPA_PRINTF_FUNC(6,7);
 
