@@ -994,6 +994,8 @@ filter_new(struct pw_context *context, const char *name,
 		res = -errno;
 		goto error_properties;
 	}
+	if ((str = pw_context_get_conf_section(context, "filter.properties")) != NULL)
+		pw_properties_update_string(props, str, strlen(str));
 
 	if (pw_properties_get(props, PW_KEY_NODE_NAME) == NULL && extra) {
 		str = pw_properties_get(extra, PW_KEY_APP_NAME);
@@ -1003,10 +1005,8 @@ filter_new(struct pw_context *context, const char *name,
 			str = name;
 		pw_properties_set(props, PW_KEY_NODE_NAME, str);
 	}
-	if (pw_properties_get(props, PW_KEY_NODE_LATENCY) == NULL) {
-		if ((str = getenv("PIPEWIRE_LATENCY")) != NULL)
-			pw_properties_set(props, PW_KEY_NODE_LATENCY, str);
-	}
+	if ((str = getenv("PIPEWIRE_LATENCY")) != NULL)
+		pw_properties_set(props, PW_KEY_NODE_LATENCY, str);
 
 	spa_hook_list_init(&impl->hooks);
 	this->properties = props;
