@@ -101,6 +101,7 @@ static void node_deactivate(struct pw_impl_node *this)
 static void add_node(struct pw_impl_node *this, struct pw_impl_node *driver)
 {
 	struct pw_node_activation_state *dstate, *nstate;
+	struct pw_node_target *t;
 
 	if (this->exported)
 		return;
@@ -114,21 +115,29 @@ static void add_node(struct pw_impl_node *this, struct pw_impl_node *driver)
 	this->rt.driver_target.data = driver;
 	spa_list_append(&this->rt.target_list, &this->rt.driver_target.link);
 
-	dstate = &this->rt.driver_target.activation->state[0];
-	dstate->required++;
-
 	spa_list_append(&driver->rt.target_list, &this->rt.target.link);
 	nstate = &this->rt.activation->state[0];
-	nstate->required++;
+	if (!this->rt.target.active) {
+		nstate->required++;
+		this->rt.target.active = true;
+	}
 
-	pw_log_trace(NAME" %p: driver state:%p pending:%d/%d, node state:%p pending:%d/%d",
-			this, dstate, dstate->pending, dstate->required,
-			nstate, nstate->pending, nstate->required);
+	spa_list_for_each(t, &this->rt.target_list, link) {
+		dstate = &t->activation->state[0];
+		if (!t->active) {
+			dstate->required++;
+			t->active = true;
+		}
+		pw_log_trace(NAME" %p: driver state:%p pending:%d/%d, node state:%p pending:%d/%d",
+				this, dstate, dstate->pending, dstate->required,
+				nstate, nstate->pending, nstate->required);
+	}
 }
 
 static void remove_node(struct pw_impl_node *this)
 {
 	struct pw_node_activation_state *dstate, *nstate;
+	struct pw_node_target *t;
 
 	if (this->exported)
 		return;
@@ -137,17 +146,27 @@ static void remove_node(struct pw_impl_node *this)
 			this, this->rt.driver_target.data,
 			this->rt.driver_target.activation, this->rt.activation);
 
-	spa_list_remove(&this->rt.driver_target.link);
-	dstate = &this->rt.driver_target.activation->state[0];
-	dstate->required--;
-
 	spa_list_remove(&this->rt.target.link);
-	nstate = &this->rt.activation->state[0];
-	nstate->required--;
 
-	pw_log_trace(NAME" %p: driver state:%p pending:%d/%d, node state:%p pending:%d/%d",
-			this, dstate, dstate->pending, dstate->required,
-			nstate, nstate->pending, nstate->required);
+	nstate = &this->rt.activation->state[0];
+	if (this->rt.target.active) {
+		nstate->required--;
+		this->rt.target.active = false;
+	}
+
+	spa_list_for_each(t, &this->rt.target_list, link) {
+		dstate = &t->activation->state[0];
+		if (t->active) {
+			dstate->required--;
+			t->active = false;
+		}
+		pw_log_trace(NAME" %p: driver state:%p pending:%d/%d, node state:%p pending:%d/%d",
+				this, dstate, dstate->pending, dstate->required,
+				nstate, nstate->pending, nstate->required);
+	}
+	spa_list_remove(&this->rt.driver_target.link);
+
+	this->rt.driver_target.node = NULL;
 }
 
 static int
@@ -195,8 +214,9 @@ static int start_node(struct pw_impl_node *this)
 
 	pw_log_debug(NAME" %p: start node", this);
 
-	res = spa_node_send_command(this->node,
-				    &SPA_NODE_COMMAND_INIT(SPA_NODE_COMMAND_Start));
+	if (!(this->driving && this->driver))
+		res = spa_node_send_command(this->node,
+			&SPA_NODE_COMMAND_INIT(SPA_NODE_COMMAND_Start));
 
 	if (res < 0)
 		pw_log_error("(%s-%u) start node error %d: %s", this->name, this->info.id,
@@ -319,6 +339,9 @@ static void node_update_state(struct pw_impl_node *node, enum pw_node_state stat
 	switch (state) {
 	case PW_NODE_STATE_RUNNING:
 		pw_loop_invoke(node->data_loop, do_node_add, 1, NULL, 0, true, node);
+		if (node->driving && node->driver)
+			spa_node_send_command(node->node,
+				&SPA_NODE_COMMAND_INIT(SPA_NODE_COMMAND_Start));
 		break;
 	default:
 		break;
@@ -707,15 +730,26 @@ static int
 do_move_nodes(struct spa_loop *loop,
 		bool async, uint32_t seq, const void *data, size_t size, void *user_data)
 {
-	struct impl *src = user_data;
+	struct impl *impl = user_data;
 	struct pw_impl_node *driver = *(struct pw_impl_node **)data;
-	struct pw_impl_node *this = &src->this;
+	struct pw_impl_node *node = &impl->this;
+	int res;
 
-	pw_log_trace(NAME" %p: driver:%p->%p", this, this->driver_node, driver);
+	pw_log_trace(NAME" %p: driver:%p->%p", node, node->driver_node, driver);
 
-	if (this->source.loop != NULL) {
-		remove_node(this);
-		add_node(this, driver);
+	if ((res = spa_node_set_io(node->node,
+		    SPA_IO_Position,
+		    &driver->rt.activation->position,
+		    sizeof(struct spa_io_position))) < 0) {
+		pw_log_debug(NAME" %p: set position: %s", node, spa_strerror(res));
+	}
+
+	pw_log_trace(NAME" %p: set position %p", node, &driver->rt.activation->position);
+	node->rt.position = &driver->rt.activation->position;
+
+	if (node->source.loop != NULL) {
+		remove_node(node);
+		add_node(node, driver);
 	}
 	return 0;
 }
@@ -732,7 +766,6 @@ int pw_impl_node_set_driver(struct pw_impl_node *node, struct pw_impl_node *driv
 {
 	struct impl *impl = SPA_CONTAINER_OF(node, struct impl, this);
 	struct pw_impl_node *old = node->driver_node;
-	int res;
 
 	if (driver == NULL)
 		driver = node;
@@ -754,21 +787,12 @@ int pw_impl_node_set_driver(struct pw_impl_node *node, struct pw_impl_node *driv
 
 	node->driver_node = driver;
 
-	pw_impl_node_emit_driver_changed(node, old, driver);
-
-	if ((res = spa_node_set_io(node->node,
-		    SPA_IO_Position,
-		    &driver->rt.activation->position,
-		    sizeof(struct spa_io_position))) < 0) {
-		pw_log_debug(NAME" %p: set position: %s", node, spa_strerror(res));
-	}
-
-	pw_log_trace(NAME" %p: set position %p", node, &driver->rt.activation->position);
-	node->rt.position = &driver->rt.activation->position;
-
 	pw_loop_invoke(node->data_loop,
 		       do_move_nodes, SPA_ID_INVALID, &driver, sizeof(struct pw_impl_node *),
 		       true, impl);
+
+	pw_impl_node_emit_driver_changed(node, old, driver);
+
 	return 0;
 }
 
@@ -852,7 +876,9 @@ static void check_properties(struct pw_impl_node *node)
 			uint32_t quantum_size;
 
 			node->latency = SPA_FRACTION(num, denom);
-			quantum_size = flp2((num * context->defaults.clock_rate / denom));
+			quantum_size = (num * context->defaults.clock_rate / denom);
+			if (context->defaults.clock_power_of_two_quantum)
+				quantum_size = flp2(quantum_size);
 
 			if (quantum_size != node->quantum_size) {
 				pw_log_debug(NAME" %p: latency '%s' quantum %u/%u",
@@ -871,7 +897,9 @@ static void check_properties(struct pw_impl_node *node)
 			uint32_t max_quantum_size;
 
 			node->max_latency = SPA_FRACTION(num, denom);
-			max_quantum_size = flp2((num * context->defaults.clock_rate / denom));
+			max_quantum_size = (num * context->defaults.clock_rate / denom);
+			if (context->defaults.clock_power_of_two_quantum)
+				max_quantum_size = flp2(max_quantum_size);
 
 			if (max_quantum_size != node->max_quantum_size) {
 				pw_log_debug(NAME" %p: max latency '%s' quantum %u/%u",
