@@ -60,6 +60,10 @@ static inline int memfd_create(const char *name, unsigned int flags)
 #define HAVE_MEMFD_CREATE 1
 #endif
 
+#ifdef __FreeBSD__
+#define MAP_LOCKED 0
+#endif
+
 /* memfd_create(2) flags */
 
 #ifndef MFD_CLOEXEC
