@@ -58,6 +58,7 @@ struct defaults {
 	uint32_t link_max_buffers;
 	unsigned int mem_warn_mlock:1;
 	unsigned int mem_allow_mlock:1;
+	unsigned int clock_power_of_two_quantum:1;
 };
 
 struct ratelimit {
@@ -513,6 +514,7 @@ struct pw_node_target {
 	struct pw_node_activation *activation;
 	int (*signal) (void *data);
 	void *data;
+	unsigned int active:1;
 };
 
 struct pw_node_activation {
